#include <algorithm>
#include <csignal>
#include <cstdio>
#include <cstdlib>
#include <vector>
#include <cmath>
#include <queue>
#include <random>
#include <unistd.h>
#include <algorithm>
using ll = long long;

namespace {
	std::vector<int> CountList[20];
} // namespace

void wrong(int num) {
	fprintf(stderr, "translate:wrong\n");
	fprintf(stderr, "Wrong Answer [%d]\n", num);
	fprintf(stdout, "%.2lf\n", 0.0);
	exit(0);
}

void ok(int L) {
	int sc = 0;
	if (L == 0) sc = 0;
	else if (L <   2) sc = 10;
	else if (L <   3) sc = 15;
	else if (L <   4) sc = 19;
	else if (L <  27) sc = 23 + 2 * (L -  4);
	else if (L <  37) sc = 69 + 3 * (L - 27) / 2;
	else if (L == 37) sc = 84;
	else if (L == 38) sc = 86;
	else if (L == 39) sc = 88;
	else if (L == 40) sc = 90;
	else if (L == 41) sc = 93;
	else if (L == 42) sc = 96;
	else sc = 100;
	
	double score = sc;
	score /= 100.0;
	if (sc == 100) fprintf(stderr, "translate:success\n");
	else fprintf(stderr, "translate:partial\n");
	fprintf(stderr, "Accepted\n");
	fprintf(stdout, "%.2lf\n", score);
}

std::vector<std::vector<int>> Bitaro(std::vector<std::vector<int>> T, int X, int Y, int casenum) {
	// Generator 1
	if (casenum % 26 <= 7) {
		int idx1 = ((casenum % 26) / 1) % 2;
		int idx2 = ((casenum % 26) / 2) % 2;
		int idx3 = ((casenum % 26) / 4) % 2;
		for (int i = 0; i < 8; i++) {
			for (int j = 0; j < 8; j++) {
				if (i == X && j == Y) T[i][j] = idx1;
				if (i != X && j == Y) T[i][j] = idx2;
				if (i == X && j != Y) T[i][j] = idx3;
			}
		}
	}
	
	// Generator 2
	else if (casenum % 26 <= 9) {
		int idx1 = (casenum % 26) - 8;
		for (int i = 0; i < 8; i++) {
			for (int j = 0; j < 8; j++) {
				if (T[i][j] != 2) continue;
				T[i][j] = (i + j + idx1) % 2;
			}
		}
	}
	
	// Generator 3
	else {
		int target = (casenum % 26) - 10;
		int idx = (12345 * casenum + 252525) % CountList[target].size();
		int mask = CountList[target][idx];
		int cnt = 0;
		for (int i = 0; i < 8; i++) {
			for (int j = 0; j < 8; j++) {
				if (T[i][j] != 2) continue;
				T[i][j] = (mask / (1 << cnt)) % 2;
				cnt += 1;
			}
		}
	}
	return T;
}

int main(int argc, char** argv) {
	int CurrentMaxL = 43;
	char buf[100009];
	if (argc < 5) {
		fprintf(stderr, "Usage: %s <out0> <in0> <out1> <in1>\n", argv[0]);
		return 1;
	}
	
	// Init
	for (int i = 0; i < 32768; i++) {
		int Popcnt = 0;
		for (int j = 0; j < 15; j++) {
			if ((i & (1 << j)) != 0) Popcnt += 1;
		}
		CountList[Popcnt].push_back(i);
	}
	
	// open pipe
	signal(SIGPIPE, SIG_IGN);
	FILE* out0 = fopen(argv[1], "r");
	FILE* in0 = fopen(argv[2], "w");
	FILE* out1 = fopen(argv[3], "r");
	FILE* in1 = fopen(argv[4], "w");
	
	// Read Queries
	int Queries;
	if (scanf("%d", &Queries) != 1) {
		fprintf(stderr, "manager failed to read testcase");
		wrong(0);
	}
	
	// Prepare
	char prepare[10];
	fprintf(in0, "Start\n"); fflush(in0);
	fprintf(in1, "Start\n"); fflush(in1);
	fprintf(in0, "%d\n", Queries); fflush(in0);
	fprintf(in1, "%d\n", Queries); fflush(in1);
	if (fscanf(out0, "%s", prepare) != 1) {
		fprintf(in0, "WA\n"); fflush(in0);
		fprintf(in1, "WA\n"); fflush(in1);
		wrong(0);
	}
	if (fscanf(out1, "%s", prepare) != 1) {
		fprintf(in0, "WA\n"); fflush(in0);
		fprintf(in1, "WA\n"); fflush(in1);
		wrong(0);
	}
	
	for (int challenge = 1; challenge <= Queries; challenge++) {
		// Read Testcase
		int X, Y, N; std::string S;
		if (scanf("%d%d%d", &X, &Y, &N) != 3) {
			fprintf(stderr, "manager failed to read testcase");
			wrong(0);
		}
		char tmp;
		if (scanf("%c", &tmp) != 1) {
			fprintf(stderr, "Error while reading input\n");
			exit(1);
		}
		for (int i = 0; i < N; i++) {
			if (scanf("%c", &tmp) != 1) {
				fprintf(stderr, "Error while reading input\n");
				exit(1);
			}
			S += tmp;
		}
		
		// Step #1A. Write to Anna
		fprintf(in0, "Success\n"); fflush(in0);
		fprintf(in0, "%d %d %d\n", X, Y, N); fflush(in0);
		for (int i = 0; i < N; i++) {
			fprintf(in0, "%c", S[i]);
		}
		fprintf(in0, "\n"); fflush(in0);
		
		// Step #1B. Read Error Code from Anna
		int err0;
		if (fscanf(out0, "%d", &err0) != 1) {
			fprintf(in1, "WA\n"); fflush(in1);
			wrong(0);
		}
		if (err0 != 0) {
			fprintf(in1, "WA\n"); fflush(in1);
			wrong(err0);
		}
		
		// Step #1C. Read Data from Anna
		std::vector<std::vector<int>> T(8, std::vector<int>(8, 0));
		for (int i = 0; i < 8; i++) {
			for (int j = 0; j < 8; j++) {
				if (fscanf(out0, "%d", &T[i][j]) != 1) {
					fprintf(in1, "WA\n"); fflush(in1);
					wrong(0);
				}
			}
		}
		T = Bitaro(T, X, Y, challenge);
		
		// Step #2A. Write to Bruno
		fprintf(in1, "Success\n"); fflush(in1);
		fprintf(in1, "%d\n", N);
		for (int i = 0; i < 8; i++) {
			for (int j = 0; j < 8; j++) {
				if (j >= 1) fprintf(in1, " ");
				fprintf(in1, "%d", T[i][j]);
			}
			fprintf(in1, "\n"); fflush(in1);
		}
		
		// Step #2B. Read Error Code from Bruno
		int err1;
		if (fscanf(out1, "%d", &err1) != 1) {
			fprintf(in0, "WA\n"); fflush(in0);
			wrong(0);
		}
		if (err1 != 0) {
			fprintf(in0, "WA\n"); fflush(in0);
			wrong(err1);
		}
		
		// Step #2C. Read Data from Bruno
		int StrSize; std::string String = "";
		if (fscanf(out1, "%d", &StrSize) != 1) {
			fprintf(in0, "WA\n"); fflush(in0);
			wrong(0);
		}
		for (int i = 0; i < StrSize + 5; i++) buf[i] = 0;
		if (fscanf(out1, "%s", buf) != 1) {
			fprintf(in0, "WA\n"); fflush(in0);
			wrong(0);
		}
		for (int i = 1; i < StrSize; i++) String += buf[i];
		
		// Score Update
		if (S != String) {
			CurrentMaxL = std::min(CurrentMaxL, N - 1);
		}
	}
	
	// Output The Score
	ok(CurrentMaxL);
	return 0;
}
